CREATE TABLE WSI_SQL_QUERY (
    PK PK_TYPE NOT NULL,
    OID ID_TYPE NOT NULL UNIQUE,
    MOD_TIME TIME_T,
    NAME VALID_GENERAL_NAME COLLATE ISO8859_1,
    DESCRIPTION LONG_TEXT,
    TARGET_CLASS_NAME GENERAL_NAME COLLATE ISO8859_1,
    WHERE_CLAUSE LONG_TEXT,
    QUERY_ARGS FOREIGN_KEY_ID,
    QUERY_ARGS_SC SELECT_CRITERIA COLLATE ISO8859_1,
    QUERY_GROUPS FOREIGN_KEY_ID,
    QUERY_GROUPS_SC SELECT_CRITERIA COLLATE ISO8859_1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE WSI_SQL_ARG (
    PK PK_TYPE NOT NULL,
    NAME VALID_GENERAL_NAME COLLATE ISO8859_1,
    ARG_TYPE VALID_GENERAL_NAME COLLATE ISO8859_1);

CREATE TABLE WSI_SQL_ARG_LINKS (
    PK PK_TYPE NOT NULL,
    SRC_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    DEST_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    SRC_ATTR_REF FOREIGN_KEY_ID,
    DEST_ATTR_REF FOREIGN_KEY_ID,
    SRC_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_NAME_KEY GENERAL_NAME COLLATE ISO8859_1,
    DEST_SEQ_KEY INDEX_TYPE);

/* A table for domain groups */
CREATE TABLE WSI_QUERY_GROUPS (
    PK PK_TYPE NOT NULL,
    OID ID_TYPE NOT NULL UNIQUE,
    MOD_TIME TIME_T,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    DESCRIPTION LONG_TEXT,
    QUERY_MEMBERS FOREIGN_KEY_ID,
    QUERY_MEMBERS_SC SELECT_CRITERIA COLLATE ISO8859_1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* domain group links */
CREATE TABLE WSI_QUERY_GRP_LINKS (
    PK PK_TYPE NOT NULL,
    SRC_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    DEST_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    SRC_ATTR_REF FOREIGN_KEY_ID,
    DEST_ATTR_REF FOREIGN_KEY_ID,
    SRC_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    BI_SRC_REF FOREIGN_KEY_ID,
    BI_DEST_REF FOREIGN_KEY_ID,
    BI_SRC_REF_CL CLASS_NAME COLLATE ISO8859_1,
    BI_DEST_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_NAME_KEY GENERAL_NAME COLLATE ISO8859_1,
    DEST_SEQ_KEY INDEX_TYPE,
    DEST_EXT_SEQ_KEY INDEX_TYPE);

ALTER TABLE WSI_SQL_QUERY ADD CONSTRAINT PK_SQL_QUERY PRIMARY KEY(PK);
ALTER TABLE WSI_SQL_ARG ADD CONSTRAINT PK_SQL_ARG PRIMARY KEY(PK);
ALTER TABLE WSI_SQL_ARG_LINKS ADD CONSTRAINT PK_SQL_ARG_LINKS PRIMARY KEY(PK);
ALTER TABLE WSI_QUERY_GROUPS ADD CONSTRAINT PK_QUERY_GROUPS PRIMARY KEY(PK);
ALTER TABLE WSI_QUERY_GRP_LINKS ADD CONSTRAINT PK_QUERY_GRP_LINKS PRIMARY KEY(PK);

CREATE UNIQUE DESCENDING INDEX IDX_SQL_QUERY ON WSI_SQL_QUERY(PK);
CREATE UNIQUE DESCENDING INDEX IDX_SQL_ARG ON WSI_SQL_ARG(PK);
CREATE UNIQUE DESCENDING INDEX IDX_SQL_ARG_LINKS ON WSI_SQL_ARG_LINKS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_QUERY_GROUPS ON WSI_QUERY_GROUPS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_QUERY_GRP_LINKS ON WSI_QUERY_GRP_LINKS(PK);

ALTER TABLE WSI_SQL_QUERY ADD CONSTRAINT FK_SQL_QUERY_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE WSI_QUERY_GROUPS ADD CONSTRAINT FK_QUERY_GROUPS_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);

CREATE SEQUENCE WSI_SQL_QUERY;
CREATE SEQUENCE WSI_SQL_ARG;
CREATE SEQUENCE WSI_SQL_ARG_LINKS;
CREATE SEQUENCE WSI_QUERY_GROUPS;
CREATE SEQUENCE WSI_QUERY_GRP_LINKS;

ALTER TABLE WSI_SQL_ARG_LINKS ADD CONSTRAINT FK_SQL_ARG_LINKS_DEST FOREIGN KEY (DEST_ATTR_REF) REFERENCES WSI_SQL_ARG (PK);
ALTER TABLE WSI_SQL_ARG_LINKS ADD CONSTRAINT FK_SQL_ARG_LINKS_SRC FOREIGN KEY (SRC_ATTR_REF) REFERENCES WSI_SQL_QUERY (PK);
ALTER TABLE WSI_QUERY_GRP_LINKS ADD CONSTRAINT FK_QUERY_GRP_LINKS_DEST FOREIGN KEY (BI_DEST_REF) REFERENCES WSI_SQL_QUERY (PK);
ALTER TABLE WSI_QUERY_GRP_LINKS ADD CONSTRAINT FK_QUERY_GRP_LINKS_SRC FOREIGN KEY (BI_SRC_REF) REFERENCES WSI_QUERY_GROUPS (PK);


CREATE UNIQUE INDEX IDX_SQL_QUERY_NAME ON WSI_SQL_QUERY (NAME);
CREATE UNIQUE INDEX IDX_QUERY_GROUPS_NAME ON WSI_QUERY_GROUPS (NAME);