DROP INDEX IDX_QUERY_GROUPS_NAME;
DROP INDEX IDX_CLASS_CAP;
ALTER TABLE WSI_CAPABILITY DROP CAPABILITY;
ALTER TABLE WSI_CAPABILITY ADD RIGHTS RIGHTS_TYPE;

ALTER TABLE WSI_CAPABILITY ADD ROLE_ID ID_TYPE;
ALTER TABLE WSI_CAPABILITY ALTER COLUMN TARGET_CLASS_NAME TO TARGET_CLASS;

CREATE INDEX IDX_CLASS_CAP ON WSI_CAPABILITY(TARGET_CLASS,RIGHTS);

ALTER TABLE WSI_ACCESS_CONTROL DROP RIGHTS;
ALTER TABLE WSI_ACCOUNTS DROP MDATA;
ALTER TABLE WSI_ACCOUNTS DROP PASS;

/* Tables for access IDs */

CREATE TABLE WSI_USER_ACCESS_CONTROL (
    PK PK_TYPE NOT NULL,
    OID ID_TYPE NOT NULL UNIQUE,
    MOD_TIME TIME_T,    
    ACCESS_ID ID_TYPE,
    DOMAIN_ID ID_TYPE,
    DOMAIN_REF FOREIGN_KEY_ID,
    DOMAIN_REF_CL SELECT_CRITERIA COLLATE ISO8859_1,
    ACCOUNT_REF FOREIGN_KEY_ID,
    ACCOUNT_REF_CL SELECT_CRITERIA COLLATE ISO8859_1);

ALTER TABLE WSI_USER_ACCESS_CONTROL ADD CONSTRAINT PK_USER_ACCESS_CONTROL PRIMARY KEY (PK);
CREATE UNIQUE DESCENDING INDEX IDX_USER_ACCESS_CONTROL ON WSI_USER_ACCESS_CONTROL(PK);

CREATE SEQUENCE WSI_USER_ACCESS_CONTROL;


/* A table for accounts */

CREATE TABLE WSI_USER_ACCOUNTS (
    PK PK_TYPE NOT NULL,
    OID ID_TYPE NOT NULL UNIQUE,
    MOD_TIME TIME_T,
    NAME VALID_GENERAL_NAME COLLATE ISO8859_1,
    ENABLED BOOL,
    DESCRIPTION LONG_TEXT,
    PASS GENERAL_NAME COLLATE ISO8859_1,
    MIN_CREATE_RIGHTS GENERAL_NAME COLLATE ISO8859_1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

ALTER TABLE WSI_USER_ACCOUNTS ADD CONSTRAINT PK_USER_ACCOUNTS PRIMARY KEY (PK);
CREATE UNIQUE DESCENDING INDEX IDX_USER_ACCOUNTS ON WSI_USER_ACCOUNTS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_USER_ACCOUNT_NAME ON WSI_USER_ACCOUNTS(NAME);

CREATE SEQUENCE WSI_USER_ACCOUNTS;

ALTER TABLE WSI_USER_ACCOUNTS ADD CONSTRAINT FK_USER_ACCOUNTS FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);

CREATE UNIQUE ASCENDING INDEX IDX_CAPABILITY_RIGHTS ON WSI_CAPABILITY(ROLE_ID,TARGET_CLASS,RIGHTS);

/* A table for roles */

CREATE TABLE WSI_ROLES (
    PK PK_TYPE NOT NULL,
    OID ID_TYPE NOT NULL UNIQUE,
    MOD_TIME TIME_T,
    DOMAIN_REF FOREIGN_KEY_ID,
    DOMAIN_REF_CL CLASS_NAME COLLATE ISO8859_1,
    CAPABILITIES  FOREIGN_KEY_ID,
    CAPABILITIES_SC SELECT_CRITERIA COLLATE ISO8859_1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

ALTER TABLE WSI_ROLES ADD CONSTRAINT PK_ROLES PRIMARY KEY (PK);
CREATE UNIQUE DESCENDING INDEX IDX_ROLES ON WSI_ROLES(PK);

CREATE SEQUENCE WSI_ROLES;

ALTER TABLE WSI_ROLES ADD CONSTRAINT FK_ROLES_DOMAIN FOREIGN KEY (DOMAIN_REF) REFERENCES WSI_DOMAINS (PK);
ALTER TABLE WSI_ROLES ADD CONSTRAINT FK_ROLES_SEC FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE WSI_USER_ACCESS_CONTROL ADD CONSTRAINT FK_USER_ACCESS_ACCOUNT FOREIGN KEY (ACCOUNT_REF) REFERENCES WSI_USER_ACCOUNTS (PK);
ALTER TABLE WSI_USER_ACCESS_CONTROL ADD CONSTRAINT FK_USER_ACCESS_DOMAIN FOREIGN KEY (DOMAIN_REF) REFERENCES WSI_ROLES (PK);

CREATE TABLE WSI_ROLE_LINKS (
    PK PK_TYPE NOT NULL,
    SRC_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    DEST_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    SRC_ATTR_REF FOREIGN_KEY_ID,
    DEST_ATTR_REF FOREIGN_KEY_ID,
    SRC_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_NAME_KEY GENERAL_NAME COLLATE ISO8859_1,
    DEST_SEQ_KEY INDEX_TYPE);

ALTER TABLE WSI_ROLE_LINKS ADD CONSTRAINT PK_ROLES_LINKS PRIMARY KEY (PK);
CREATE UNIQUE DESCENDING INDEX IDX_ROLE_LINKS ON WSI_ROLE_LINKS(PK);

CREATE SEQUENCE WSI_ROLE_LINKS;

ALTER TABLE WSI_ROLE_LINKS ADD CONSTRAINT FK_ROLE_LINKS_SRC FOREIGN KEY (SRC_ATTR_REF) REFERENCES WSI_ROLES (PK);
ALTER TABLE WSI_ROLE_LINKS ADD CONSTRAINT FK_ROLE_LINKS_DST FOREIGN KEY (DEST_ATTR_REF) REFERENCES WSI_CAPABILITY (PK);

CREATE DOMAIN DOM_TYPE AS INTEGER
DEFAULT 0;

ALTER TABLE WSI_DOMAIN_ID ADD DOMAIN_TYPE DOM_TYPE;
ALTER TABLE WSI_DOMAINS ADD DOMAIN_TYPE DOM_TYPE;

ALTER TABLE WSI_RULE4 ADD ROLES_RESULT LONG_TEXT;
ALTER TABLE WSI_RULE6 ADD ROLES_RESULT LONG_TEXT;
ALTER TABLE WSI_RULE4 ALTER COLUMN RESULT TO DOMAINS_RESULT;
ALTER TABLE WSI_RULE6 ALTER COLUMN RESULT TO DOMAINS_RESULT;

CREATE TABLE WSI_REP_PEERS (
    PK PK_TYPE NOT NULL,
    OID ID_TYPE NOT NULL UNIQUE,
    MOD_TIME TIME_T,
    DESCRIPTION LONG_TEXT,
    TARGET GENERAL_NAME COLLATE ISO8859_1,
    CLASSES LONG_TEXT,
    CONSOLE_USER GENERAL_NAME COLLATE ISO8859_1,
    CONSOLE_PASS GENERAL_NAME COLLATE ISO8859_1,
    ENABLED BOOL,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

ALTER TABLE WSI_REP_PEERS ADD CONSTRAINT PK_WSI_REP_PEERS PRIMARY KEY (PK);
CREATE UNIQUE DESCENDING INDEX IDX_WSI_REP_PEERS ON WSI_REP_PEERS(PK);

CREATE SEQUENCE WSI_REP_PEERS;

ALTER TABLE WSI_REP_PEERS ADD CONSTRAINT FK_WSI_REP_PEERS_SEC FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);